@@ -12,18 +12,20 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-
 import pecan
 from pecan import rest
 
 from mistral import exceptions as exc
+from mistral.openstack.common import log as logging
+
+
+LOG = logging.getLogger(__name__)
 
 
 class SpecValidationController(rest.RestController):
 
     def __init__(self, parser):
         super(SpecValidationController, self).__init__()
-
         self._parse_func = parser
 
     @pecan.expose('json')
@@ -32,8 +34,8 @@ class SpecValidationController(rest.RestController):
         definition = pecan.request.text
 
         try:
-            self._parse_func(definition, validate=True)
+            self._parse_func(definition)
         except exc.DSLParsingException as e:
-            return {'valid': False, 'error': str(e)}
+            return {'valid': False, 'error': e.message}
 
         return {'valid': True}
