@@ -13,22 +13,15 @@
 
 """Starter script for mistral-db-manage."""
 
-# NOTE(amorin)
-# Hardcode the threading backend to avoid using eventlet until this will
-# eventually become the default
-import oslo_service.backend as service_backend
-service_backend.init_backend(service_backend.BackendType.THREADING)
-
 import os
 
 from alembic import command as alembic_cmd
 from alembic import config as alembic_cfg
 from alembic import util as alembic_u
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_utils import importutils
-import sys
+from oslo.config import cfg
+import six
 
+from mistral.openstack.common import importutils
 from mistral.services import action_manager
 from mistral.services import workflows
 
@@ -39,14 +32,13 @@ importutils.try_import('mistral.api.app')
 
 
 CONF = cfg.CONF
-LOG = logging.getLogger(__name__)
 
 
 def do_alembic_command(config, cmd, *args, **kwargs):
     try:
         getattr(alembic_cmd, cmd)(config, *args, **kwargs)
     except alembic_u.CommandError as e:
-        alembic_u.err(str(e))
+        alembic_u.err(six.text_type(e))
 
 
 def do_check_migration(config, _cmd):
@@ -75,23 +67,16 @@ def do_stamp(config, cmd):
 
 
 def do_populate(config, cmd):
-    LOG.info("Populating db")
     action_manager.sync_db()
     workflows.sync_db()
 
 
-def do_populate_actions(config, cmd):
-    LOG.info("Populating actions db")
-    action_manager.sync_db()
-
-
 def do_revision(config, cmd):
     do_alembic_command(
         config, cmd,
         message=CONF.command.message,
         autogenerate=CONF.command.autogenerate,
-        sql=CONF.command.sql,
-        rev_id=CONF.command.rev_id
+        sql=CONF.command.sql
     )
 
 
@@ -109,9 +94,6 @@ def add_command_parsers(subparsers):
     parser = subparsers.add_parser('populate')
     parser.set_defaults(func=do_populate)
 
-    parser = subparsers.add_parser('populate_actions')
-    parser.set_defaults(func=do_populate_actions)
-
     parser = subparsers.add_parser('stamp')
     parser.add_argument('--sql', action='store_true')
     parser.add_argument('revision', nargs='?')
@@ -119,7 +101,6 @@ def add_command_parsers(subparsers):
 
     parser = subparsers.add_parser('revision')
     parser.add_argument('-m', '--message')
-    parser.add_argument('--rev-id', dest='rev_id')
     parser.add_argument('--autogenerate', action='store_true')
     parser.add_argument('--sql', action='store_true')
     parser.set_defaults(func=do_revision)
@@ -132,14 +113,6 @@ command_opt = cfg.SubCommandOpt('command',
 
 CONF.register_cli_opt(command_opt)
 
-# To Keep backwards compatibility we need to accept mapping path
-# from mistral-extra if present
-try:
-    import mistral_extra.config as extra_conf
-    CONF.register_cli_opt(extra_conf.os_actions_mapping_path)
-except ImportError:
-    LOG.debug("Mistral-extra not installed")
-
 
 def main():
     config = alembic_cfg.Config(
@@ -151,12 +124,9 @@ def main():
     )
     # attach the Mistral conf to the Alembic conf
     config.mistral_config = CONF
-    logging.register_options(CONF)
 
     CONF(project='mistral')
-    logging.setup(CONF, 'Mistral')
     CONF.command.func(config, CONF.command.name)
 
-
 if __name__ == '__main__':
-    sys.exit(main())
+    main()
