@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+#
 # Copyright 2013 - Mirantis, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,43 +15,47 @@
 #    limitations under the License.
 
 import mock
+from oslo.config import cfg
 import pecan
 import pecan.testing
 from webtest import app as webtest_app
 
-from mistral.api import app as pecan_app
-from mistral.services import periodic
-from mistral.tests.unit import base
-from mistral.tests.unit.mstrlfixtures import policy_fixtures
+from mistral.tests import base
+
+# Disable authentication for functional tests.
+cfg.CONF.set_default('auth_enable', False, group='pecan')
+
+__all__ = ['FunctionalTest']
 
 
-class APITest(base.DbTestCase):
+class FunctionalTest(base.DbTestCase):
+    """Used for functional tests where you need to test your
+    literal application and its integration with the framework.
+    """
 
     def setUp(self):
-        super(APITest, self).setUp()
+        super(FunctionalTest, self).setUp()
 
-        self.override_config('auth_enable', False, group='pecan')
-        self.override_config('enabled', False, group='cron_trigger')
+        pecan_opts = cfg.CONF.pecan
 
-        self.app = pecan.testing.load_test_app(
-            dict(pecan_app.get_pecan_config())
-        )
+        self.app = pecan.testing.load_test_app({
+            'app': {
+                'root': pecan_opts.root,
+                'modules': pecan_opts.modules,
+                'debug': pecan_opts.debug,
+                'auth_enable': False
+            }
+        })
+        self.addCleanup(pecan.set_config, {}, overwrite=True)
+        self.addCleanup(cfg.CONF.set_default, 'auth_enable', False,
+                        group='pecan')
 
-        # Adding cron trigger thread clean up explicitly in case if
-        # new tests will provide an alternative configuration for pecan
-        # application.
-        self.addCleanup(periodic.stop_all_periodic_tasks)
-
-        # Make sure the api get the correct context.
-        self.patch_ctx = mock.patch(
-            'mistral.context.MistralContext.from_environ'
-        )
+        # make sure the api get the correct context.
+        self.patch_ctx = mock.patch('mistral.context.context_from_headers')
         self.mock_ctx = self.patch_ctx.start()
         self.mock_ctx.return_value = self.ctx
         self.addCleanup(self.patch_ctx.stop)
 
-        self.policy = self.useFixture(policy_fixtures.PolicyFixture())
-
     def assertNotFound(self, url):
         try:
             self.app.get(url, headers={'Accept': 'application/json'})
