@@ -14,15 +14,17 @@
 
 from eventlet import corolocal
 from eventlet import semaphore
-from oslo_config import cfg
-import testtools
+from oslo.config import cfg
 
+from mistral.actions import base as action_base
 from mistral.db.v2 import api as db_api
+from mistral.openstack.common import log as logging
+from mistral.services import action_manager as a_m
 from mistral.services import workflows as wf_service
 from mistral.tests.unit.engine import base
 from mistral.workflow import states
-from mistral_lib import actions as actions_base
 
+LOG = logging.getLogger(__name__)
 
 # Use the set_default method to set value otherwise in certain test cases
 # the change in value is not permanent.
@@ -45,9 +47,9 @@ wf:
 
   tasks:
     task1:
-      action: test.block
+      action: std.block
       publish:
-        result: <% task(task1).result %>
+        result: <% $.task1 %>
 """
 
 WF_SHORT_ACTION = """
@@ -61,14 +63,14 @@ wf:
     The idea is to use action that runs faster than engine.start_workflow().
     And we need to check that engine handles this situation as well. This was
     a situation previously that led to a race condition in engine, method
-    on_action_complete() was called while DB transaction in start_workflow()
-    was still active (not committed yet).
+    on_task_result() was called while DB transaction in start_workflow() was
+    still active (not committed yet).
     To emulate a short action we use a workflow with two start tasks so they
     run both in parallel on the first engine iteration when we call method
     start_workflow(). First task has a short action that just returns a
     predefined result and the second task blocks until the test explicitly
     unblocks it. So the first action will always end before start_workflow()
-    method ends.
+    methods ends.
 
   output:
     result: <% $.result %>
@@ -77,17 +79,17 @@ wf:
     task1:
       action: std.echo output=1
       publish:
-        result: <% task(task1).result %>
+        result: <% $.task1 %>
 
     task2:
-      action: test.block
+      action: std.block
 """
 
 ACTION_SEMAPHORE = None
 TEST_SEMAPHORE = None
 
 
-class BlockingAction(actions_base.Action):
+class BlockingAction(action_base.Action):
     def __init__(self):
         pass
 
@@ -99,7 +101,7 @@ class BlockingAction(actions_base.Action):
     def wait_for_test():
         ACTION_SEMAPHORE.acquire()
 
-    def run(self, context):
+    def run(self):
         self.unblock_test()
         self.wait_for_test()
 
@@ -111,9 +113,9 @@ class BlockingAction(actions_base.Action):
         pass
 
 
-class EngineActionRaceConditionTest(base.EngineTestCase):
+class LongActionTest(base.EngineTestCase):
     def setUp(self):
-        super(EngineActionRaceConditionTest, self).setUp()
+        super(LongActionTest, self).setUp()
 
         global ACTION_SEMAPHORE
         global TEST_SEMAPHORE
@@ -121,7 +123,11 @@ class EngineActionRaceConditionTest(base.EngineTestCase):
         ACTION_SEMAPHORE = semaphore.Semaphore(1)
         TEST_SEMAPHORE = semaphore.Semaphore(0)
 
-        self.register_action_class('test.block', BlockingAction)
+        a_m.register_action_class(
+            'std.block',
+            '%s.%s' % (BlockingAction.__module__, BlockingAction.__name__),
+            None
+        )
 
     @staticmethod
     def block_action():
@@ -140,54 +146,37 @@ class EngineActionRaceConditionTest(base.EngineTestCase):
 
         self.block_action()
 
-        wf_ex = self.engine.start_workflow('wf')
+        wf_ex = self.engine.start_workflow('wf', None)
 
-        with db_api.transaction():
-            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-            task_execs = wf_ex.task_executions
-        self.await_task_running(task_execs[0].id)
-
-        with db_api.transaction():
-            wf_ex = db_api.get_workflow_execution(wf_ex.id)
-
-            task_execs = wf_ex.task_executions
         self.assertEqual(states.RUNNING, wf_ex.state)
-        self.assertEqual(states.RUNNING, task_execs[0].state)
+        self.assertEqual(states.RUNNING, wf_ex.task_executions[0].state)
 
         self.wait_for_action()
 
-        with db_api.transaction():
-            # Here's the point when the action is blocked but already running.
-            # Do the same check again, it should always pass.
-            wf_ex = db_api.get_workflow_execution(wf_ex.id)
-
-            task_execs = wf_ex.task_executions
+        # Here's the point when the action is blocked but already running.
+        # Do the same check again, it should always pass.
+        wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
-        self.assertEqual(states.RUNNING, task_execs[0].state)
+        self.assertEqual(states.RUNNING, wf_ex.task_executions[0].state)
 
         self.unblock_action()
 
-        self.await_workflow_success(wf_ex.id)
+        self._await(lambda: self.is_execution_success(wf_ex.id))
 
-        with db_api.transaction():
-            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-            wf_output = wf_ex.output
+        self.assertDictEqual({'result': 'test'}, wf_ex.output)
 
-        self.assertDictEqual({'result': 'test'}, wf_output)
-
-    # TODO(rakhmerov): Should periodically fail now because of poor
-    # transaction isolation support in SQLite. Requires more research
-    # to understand all the details. It's not reproducible on MySql.
-    @testtools.skip('Skip until we know how to fix it with SQLite.')
+    # TODO(rakhmerov): Should periodically fail now. Fix race condition.
     def test_short_action(self):
         wf_service.create_workflows(WF_SHORT_ACTION)
 
         self.block_action()
 
-        wf_ex = self.engine.start_workflow('wf')
+        wf_ex = self.engine.start_workflow('wf', None)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -202,12 +191,12 @@ class EngineActionRaceConditionTest(base.EngineTestCase):
             state=states.RUNNING
         )
 
-        self.await_task_success(task1_ex.id, timeout=10)
+        self._await(lambda: self.is_task_success(task1_ex.id))
 
         self.unblock_action()
 
-        self.await_task_success(task2_ex.id)
-        self.await_workflow_success(wf_ex.id)
+        self._await(lambda: self.is_task_success(task2_ex.id))
+        self._await(lambda: self.is_execution_success(wf_ex.id))
 
         task1_ex = db_api.get_task_execution(task1_ex.id)
         task1_action_ex = db_api.get_action_executions(
