@@ -1,5 +1,4 @@
 # Copyright 2015 - StackStorm, Inc.
-# Copyright 2016 - Brocade Communications Systems, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -18,11 +17,17 @@ import copy
 import yaml
 
 from mistral import exceptions as exc
+from mistral.openstack.common import log as logging
 from mistral.tests.unit.workbook.v2 import base
 from mistral import utils
+from mistral.workbook.v2 import tasks
+
+
+LOG = logging.getLogger(__name__)
 
 
 class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
+
     def test_workflow_types(self):
         tests = [
             ({'type': 'direct'}, False),
@@ -33,12 +38,9 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
 
         for wf_type, expect_error in tests:
             overlay = {'test': wf_type}
-
-            self._parse_dsl_spec(
-                add_tasks=True,
-                changes=overlay,
-                expect_error=expect_error
-            )
+            self._parse_dsl_spec(add_tasks=True,
+                                 changes=overlay,
+                                 expect_error=expect_error)
 
     def test_direct_workflow(self):
         overlay = {'test': {'type': 'direct', 'tasks': {}}}
@@ -49,80 +51,25 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
         utils.merge_dicts(overlay['test']['tasks'], {'echo': on_success})
         utils.merge_dicts(overlay['test']['tasks'], {'email': join})
 
-        wfs_spec = self._parse_dsl_spec(
-            add_tasks=True,
-            changes=overlay,
-            expect_error=False
-        )
+        wfs_spec = self._parse_dsl_spec(add_tasks=True,
+                                        changes=overlay,
+                                        expect_error=False)
 
         self.assertEqual(1, len(wfs_spec.get_workflows()))
         self.assertEqual('test', wfs_spec.get_workflows()[0].get_name())
         self.assertEqual('direct', wfs_spec.get_workflows()[0].get_type())
+        self.assertIsInstance(wfs_spec.get_workflows()[0].get_tasks(),
+                              tasks.DirectWfTaskSpecList)
 
     def test_direct_workflow_invalid_task(self):
-        overlay = {
-            'test': {
-                'type': 'direct',
-                'tasks': {}
-            }
-        }
+        overlay = {'test': {'type': 'direct', 'tasks': {}}}
         requires = {'requires': ['echo', 'get']}
 
         utils.merge_dicts(overlay['test']['tasks'], {'email': requires})
 
-        self._parse_dsl_spec(
-            add_tasks=True,
-            changes=overlay,
-            expect_error=True
-        )
-
-    def test_direct_workflow_no_start_tasks(self):
-        overlay = {
-            'test': {
-                'type': 'direct',
-                'tasks': {
-                    'task1': {'on-complete': 'task2'},
-                    'task2': {'on-complete': 'task1'}
-                }
-            }
-        }
-
-        self._parse_dsl_spec(
-            add_tasks=False,
-            changes=overlay,
-            expect_error=True
-        )
-
-    def test_direct_workflow_invalid_join(self):
-        tests = [
-            ({'task3': {'join': 2}}, False),
-            ({'task3': {'join': 5}}, True),
-            ({'task3': {'join': 1}}, False),
-            ({'task3': {'join': 'one'}}, False),
-            ({'task3': {'join': 'all'}}, False),
-            ({'task4': {'join': 'all'}}, True),
-            ({'task4': {'join': 1}}, True),
-            ({'task4': {'join': 'one'}}, True)
-        ]
-
-        for test in tests:
-            overlay = {
-                'test': {
-                    'type': 'direct',
-                    'tasks': {
-                        'task1': {'on-complete': 'task3'},
-                        'task2': {'on-complete': 'task3'}
-                    }
-                }
-            }
-
-            utils.merge_dicts(overlay['test']['tasks'], test[0])
-
-            self._parse_dsl_spec(
-                add_tasks=False,
-                changes=overlay,
-                expect_error=test[1]
-            )
+        self._parse_dsl_spec(add_tasks=True,
+                             changes=overlay,
+                             expect_error=True)
 
     def test_reverse_workflow(self):
         overlay = {'test': {'type': 'reverse', 'tasks': {}}}
@@ -130,15 +77,15 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
 
         utils.merge_dicts(overlay['test']['tasks'], {'email': require})
 
-        wfs_spec = self._parse_dsl_spec(
-            add_tasks=True,
-            changes=overlay,
-            expect_error=False
-        )
+        wfs_spec = self._parse_dsl_spec(add_tasks=True,
+                                        changes=overlay,
+                                        expect_error=False)
 
         self.assertEqual(1, len(wfs_spec.get_workflows()))
         self.assertEqual('test', wfs_spec.get_workflows()[0].get_name())
         self.assertEqual('reverse', wfs_spec.get_workflows()[0].get_type())
+        self.assertIsInstance(wfs_spec.get_workflows()[0].get_tasks(),
+                              tasks.ReverseWfTaskSpecList)
 
     def test_reverse_workflow_invalid_task(self):
         overlay = {'test': {'type': 'reverse', 'tasks': {}}}
@@ -149,21 +96,17 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
         utils.merge_dicts(overlay['test']['tasks'], {'echo': on_success})
         utils.merge_dicts(overlay['test']['tasks'], {'email': join})
 
-        self._parse_dsl_spec(
-            add_tasks=True,
-            changes=overlay,
-            expect_error=True
-        )
+        self._parse_dsl_spec(add_tasks=True,
+                             changes=overlay,
+                             expect_error=True)
 
     def test_version_required(self):
         dsl_dict = copy.deepcopy(self._dsl_blank)
         dsl_dict.pop('version', None)
 
-        exception = self.assertRaises(
-            exc.DSLParsingException,
-            self._spec_parser,
-            yaml.safe_dump(dsl_dict)
-        )
+        exception = self.assertRaises(exc.DSLParsingException,
+                                      self._spec_parser,
+                                      yaml.safe_dump(dsl_dict))
 
         self.assertIn("'version' is a required property", exception.message)
 
@@ -177,11 +120,9 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
         ]
 
         for version, expect_error in tests:
-            self._parse_dsl_spec(
-                add_tasks=True,
-                changes=version,
-                expect_error=expect_error
-            )
+            self._parse_dsl_spec(add_tasks=True,
+                                 changes=version,
+                                 expect_error=expect_error)
 
     def test_inputs(self):
         tests = [
@@ -201,12 +142,9 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
 
         for wf_input, expect_error in tests:
             overlay = {'test': wf_input}
-
-            self._parse_dsl_spec(
-                add_tasks=True,
-                changes=overlay,
-                expect_error=expect_error
-            )
+            self._parse_dsl_spec(add_tasks=True,
+                                 changes=overlay,
+                                 expect_error=expect_error)
 
     def test_outputs(self):
         tests = [
@@ -222,42 +160,13 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
 
         for wf_output, expect_error in tests:
             overlay = {'test': wf_output}
-
-            self._parse_dsl_spec(
-                add_tasks=True,
-                changes=overlay,
-                expect_error=expect_error
-            )
-
-    def test_vars(self):
-        tests = [
-            ({'vars': {'v1': 'a', 'v2': 1, 'v3': True, 'v4': None}}, False),
-            ({'vars': {'v1': '<% $.input_var1 %>'}}, False),
-            ({'vars': {'v1': '<% 1 + 2 %>'}}, False),
-            ({'vars': {'v1': '<% * %>'}}, True),
-            ({'vars': {'v1': '{{ _.input_var1 }}'}}, False),
-            ({'vars': {'v1': '{{ 1 + 2 }}'}}, False),
-            ({'vars': {'v1': '{{ * }}'}}, True),
-            ({'vars': []}, True),
-            ({'vars': 'whatever'}, True),
-            ({'vars': None}, True),
-            ({'vars': {}}, True)
-        ]
-
-        for wf_vars, expect_error in tests:
-            overlay = {'test': wf_vars}
-
-            self._parse_dsl_spec(
-                add_tasks=True,
-                changes=overlay,
-                expect_error=expect_error
-            )
+            self._parse_dsl_spec(add_tasks=True,
+                                 changes=overlay,
+                                 expect_error=expect_error)
 
     def test_tasks_required(self):
-        exception = self._parse_dsl_spec(
-            add_tasks=False,
-            expect_error=True
-        )
+        exception = self._parse_dsl_spec(add_tasks=False,
+                                         expect_error=True)
 
         self.assertIn("'tasks' is a required property", exception.message)
 
@@ -270,12 +179,9 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
 
         for wf_tasks, expect_error in tests:
             overlay = {'test': wf_tasks}
-
-            self._parse_dsl_spec(
-                add_tasks=False,
-                changes=overlay,
-                expect_error=expect_error
-            )
+            self._parse_dsl_spec(add_tasks=False,
+                                 changes=overlay,
+                                 expect_error=expect_error)
 
     def test_task_defaults(self):
         tests = [
@@ -284,10 +190,6 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
             ({'on-success': [{'email': '<% 1 %>'}, 'echo']}, False),
             ({'on-success': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
             ({'on-success': [{'email': '<% * %>'}]}, True),
-            ({'on-success': [{'email': '{{ 1 }}'}]}, False),
-            ({'on-success': [{'email': '{{ 1 }}'}, 'echo']}, False),
-            ({'on-success': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
-            ({'on-success': [{'email': '{{ * }}'}]}, True),
             ({'on-success': 'email'}, False),
             ({'on-success': None}, True),
             ({'on-success': ['']}, True),
@@ -299,10 +201,6 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
             ({'on-error': [{'email': '<% 1 %>'}, 'echo']}, False),
             ({'on-error': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
             ({'on-error': [{'email': '<% * %>'}]}, True),
-            ({'on-error': [{'email': '{{ 1 }}'}]}, False),
-            ({'on-error': [{'email': '{{ 1 }}'}, 'echo']}, False),
-            ({'on-error': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
-            ({'on-error': [{'email': '{{ * }}'}]}, True),
             ({'on-error': 'email'}, False),
             ({'on-error': None}, True),
             ({'on-error': ['']}, True),
@@ -314,10 +212,6 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
             ({'on-complete': [{'email': '<% 1 %>'}, 'echo']}, False),
             ({'on-complete': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
             ({'on-complete': [{'email': '<% * %>'}]}, True),
-            ({'on-complete': [{'email': '{{ 1 }}'}]}, False),
-            ({'on-complete': [{'email': '{{ 1 }}'}, 'echo']}, False),
-            ({'on-complete': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
-            ({'on-complete': [{'email': '{{ * }}'}]}, True),
             ({'on-complete': 'email'}, False),
             ({'on-complete': None}, True),
             ({'on-complete': ['']}, True),
@@ -337,10 +231,6 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
             ({'retry': {'count': '<% * %>', 'delay': 1}}, True),
             ({'retry': {'count': 3, 'delay': '<% 1 %>'}}, False),
             ({'retry': {'count': 3, 'delay': '<% * %>'}}, True),
-            ({'retry': {'count': '{{ 3 }}', 'delay': 1}}, False),
-            ({'retry': {'count': '{{ * }}', 'delay': 1}}, True),
-            ({'retry': {'count': 3, 'delay': '{{ 1 }}'}}, False),
-            ({'retry': {'count': 3, 'delay': '{{ * }}'}}, True),
             ({'retry': {'count': -3, 'delay': 1}}, True),
             ({'retry': {'count': 3, 'delay': -1}}, True),
             ({'retry': {'count': '3', 'delay': 1}}, True),
@@ -349,38 +239,28 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
             ({'wait-before': 1}, False),
             ({'wait-before': '<% 1 %>'}, False),
             ({'wait-before': '<% * %>'}, True),
-            ({'wait-before': '{{ 1 }}'}, False),
-            ({'wait-before': '{{ * }}'}, True),
             ({'wait-before': -1}, True),
             ({'wait-before': 1.0}, True),
             ({'wait-before': '1'}, True),
             ({'wait-after': 1}, False),
             ({'wait-after': '<% 1 %>'}, False),
             ({'wait-after': '<% * %>'}, True),
-            ({'wait-after': '{{ 1 }}'}, False),
-            ({'wait-after': '{{ * }}'}, True),
             ({'wait-after': -1}, True),
             ({'wait-after': 1.0}, True),
             ({'wait-after': '1'}, True),
             ({'timeout': 300}, False),
             ({'timeout': '<% 300 %>'}, False),
             ({'timeout': '<% * %>'}, True),
-            ({'timeout': '{{ 300 }}'}, False),
-            ({'timeout': '{{ * }}'}, True),
             ({'timeout': -300}, True),
             ({'timeout': 300.0}, True),
             ({'timeout': '300'}, True),
             ({'pause-before': False}, False),
             ({'pause-before': '<% False %>'}, False),
             ({'pause-before': '<% * %>'}, True),
-            ({'pause-before': '{{ False }}'}, False),
-            ({'pause-before': '{{ * }}'}, True),
             ({'pause-before': 'False'}, True),
             ({'concurrency': 10}, False),
             ({'concurrency': '<% 10 %>'}, False),
             ({'concurrency': '<% * %>'}, True),
-            ({'concurrency': '{{ 10 }}'}, False),
-            ({'concurrency': '{{ * }}'}, True),
             ({'concurrency': -10}, True),
             ({'concurrency': 10.0}, True),
             ({'concurrency': '10'}, True)
@@ -391,37 +271,6 @@ class WorkflowSpecValidation(base.WorkflowSpecValidationTestCase):
 
             utils.merge_dicts(overlay['test']['task-defaults'], default)
 
-            self._parse_dsl_spec(
-                add_tasks=True,
-                changes=overlay,
-                expect_error=expect_error
-            )
-
-    def test_invalid_item(self):
-        overlay = {'name': 'invalid'}
-
-        exception = self._parse_dsl_spec(changes=overlay, expect_error=True)
-
-        self.assertIn("Invalid DSL", exception.message)
-
-    def test_invalid_name(self):
-        invalid_wf = {
-            'version': '2.0',
-            'b98180ba-48a0-4e26-ab2e-50dc224f6fd1': {
-                'type': 'direct',
-                'tasks': {'t1': {'action': 'std.noop'}}
-            }
-        }
-
-        dsl_yaml = yaml.safe_dump(invalid_wf, default_flow_style=False)
-
-        exception = self.assertRaises(
-            exc.InvalidModelException,
-            self._spec_parser,
-            dsl_yaml
-        )
-
-        self.assertIn(
-            "Workflow name cannot be in the format of UUID",
-            exception.message
-        )
+            self._parse_dsl_spec(add_tasks=True,
+                                 changes=overlay,
+                                 expect_error=expect_error)
