@@ -20,6 +20,7 @@ from mistral.workflow import base
 from mistral.workflow import commands
 from mistral.workflow import data_flow
 from mistral.workflow import states
+from mistral.workflow import utils as wf_utils
 
 
 class ReverseWorkflowController(base.WorkflowController):
@@ -39,31 +40,19 @@ class ReverseWorkflowController(base.WorkflowController):
 
     __workflow_type__ = "reverse"
 
-    def _find_next_commands(self, task_ex):
-        """Finds all tasks with resolved dependencies.
-
-         This method finds all tasks with resolved dependencies and
-         returns them in the form of workflow commands.
+    def _find_next_commands(self):
+        """Finds all tasks with resolved dependencies and return them
+         in the form of workflow commands.
         """
-        cmds = super(ReverseWorkflowController, self)._find_next_commands(
-            task_ex
-        )
-
-        # TODO(rakhmerov): Adapt reverse workflow to non-locking model.
-        # 1. Task search must use task_ex parameter.
-        # 2. When a task has more than one dependency it's possible to
-        #   get into 'phantom read' phenomena and create multiple instances
-        #   of the same task. So 'unique_key' in conjunction with 'wait_flag'
-        #   must be used to prevent this.
+        cmds = super(ReverseWorkflowController, self)._find_next_commands()
 
         task_specs = self._find_task_specs_with_satisfied_dependencies()
 
         return cmds + [
             commands.RunTask(
                 self.wf_ex,
-                self.wf_spec,
                 t_s,
-                self.get_task_inbound_context(t_s)
+                self._get_task_inbound_context(t_s)
             )
             for t_s in task_specs
         ]
@@ -82,36 +71,27 @@ class ReverseWorkflowController(base.WorkflowController):
         return task_spec
 
     def _get_upstream_task_executions(self, task_spec):
-        t_specs_names = self.wf_spec.get_task_requires(task_spec) or []
-        t_execs = self._get_task_executions(name={'in': t_specs_names})
+        t_specs = [
+            self.wf_spec.get_tasks()[t_name]
+            for t_name in self._get_task_requires(task_spec)
+            or []
+        ]
 
-        return [t_ex for t_ex in t_execs if t_ex.state == states.SUCCESS]
+        return filter(
+            lambda t_e: t_e.state == states.SUCCESS,
+            wf_utils.find_task_executions(self.wf_ex, t_specs)
+        )
 
     def evaluate_workflow_final_context(self):
-        task_name = self._get_target_task_specification().get_name()
-        task_execs = self._get_task_executions(name=task_name)
-
-        # NOTE: For reverse workflow there can't be multiple
-        # executions for one task.
-        assert len(task_execs) <= 1
-
-        if len(task_execs) == 1:
-            return data_flow.evaluate_task_outbound_context(task_execs[0])
-        else:
-            return {}
-
-    def get_logical_task_state(self, task_ex):
-        # TODO(rakhmerov): Implement.
-        return base.TaskLogicalState(task_ex.state, task_ex.state_info)
-
-    def find_indirectly_affected_task_executions(self, task_name):
-        return set()
-
-    def is_error_handled_for(self, task_ex):
-        return task_ex.state != states.ERROR
+        return data_flow.evaluate_task_outbound_context(
+            wf_utils.find_task_execution(
+                self.wf_ex,
+                self._get_target_task_specification()
+            )
+        )
 
     def all_errors_handled(self):
-        return len(self._get_task_executions(state=states.ERROR)) == 0
+        return len(wf_utils.find_error_tasks(self.wf_ex)) == 0
 
     def _find_task_specs_with_satisfied_dependencies(self):
         """Given a target task name finds tasks with no dependencies.
@@ -134,10 +114,12 @@ class ReverseWorkflowController(base.WorkflowController):
         ]
 
     def _is_satisfied_task(self, task_spec):
-        if self._get_task_executions(name=task_spec.get_name()):
+        task_ex = wf_utils.find_task_execution(self.wf_ex, task_spec)
+
+        if task_ex:
             return False
 
-        if not self.wf_spec.get_task_requires(task_spec):
+        if not self._get_task_requires(task_spec):
             return True
 
         success_t_names = set()
@@ -146,9 +128,7 @@ class ReverseWorkflowController(base.WorkflowController):
             if t_ex.state == states.SUCCESS:
                 success_t_names.add(t_ex.name)
 
-        return not (
-            set(self.wf_spec.get_task_requires(task_spec)) - success_t_names
-        )
+        return not (set(self._get_task_requires(task_spec)) - success_t_names)
 
     def _build_graph(self, tasks_spec):
         graph = nx.DiGraph()
@@ -165,7 +145,7 @@ class ReverseWorkflowController(base.WorkflowController):
         return graph
 
     def _get_dependency_tasks(self, tasks_spec, task_spec):
-        dep_task_names = self.wf_spec.get_task_requires(task_spec)
+        dep_task_names = self._get_task_requires(task_spec)
 
         if len(dep_task_names) == 0:
             return []
@@ -178,3 +158,15 @@ class ReverseWorkflowController(base.WorkflowController):
                     dep_t_specs.add(t_spec)
 
         return dep_t_specs
+
+    def _get_task_requires(self, task_spec):
+        requires = set(task_spec.get_requires())
+
+        task_defaults = self.wf_spec.get_task_defaults()
+
+        if task_defaults:
+            requires |= set(task_defaults.get_requires())
+
+        requires.discard(task_spec.get_name())
+
+        return list(requires)
